import os
import time
import oommfc as oc
import discretisedfield as df


def macrospin():
    """Return a sytsem that represents a macrospin."""
    # define macro spin (i.e. one discretisation cell)
    p1 = (0, 0, 0)  # all lengths in metre
    p2 = (5e-9, 5e-9, 5e-9)
    cell = (5e-9, 5e-9, 5e-9)
    mesh = oc.Mesh(p1=p1, p2=p2, cell=cell)

    initial_m = (1, 0, 0)  # vector in x direction
    Ms = 8e6  # magnetisation saturation (A/m)
    m = df.Field(mesh, value=initial_m, norm=Ms)

    zeeman = oc.Zeeman(H=(0, 0, 5e6))  # external magnetic field (A/m)

    gamma = 2.211e5  # gyrotropic ratio
    alpha = 0.05  # Gilbert damping

    runid = "example-macrospin"
    system = oc.System(name=runid)
    system.hamiltonian = zeeman
    system.m = m
    system.dynamics = oc.Precession(gamma) + oc.Damping(alpha)

    return system



from .driver import Driver


class MinDriver(Driver):
    def _script(self, system):
        meshname = system.m.mesh.name
        systemname = system.name

        mif = "# m0 file\n"
        mif += "Specify Oxs_FileVectorField:m0file {\n"
        mif += "   atlas :atlas\n"
        mif += "   file m0.omf\n"
        mif += "}\n\n"

        mif += "# CGEvolver\n"
        mif += "Specify Oxs_CGEvolve {}\n\n"
        mif += "# MinDriver\n"
        mif += "Specify Oxs_MinDriver {\n"
        mif += "  evolver Oxs_CGEvolve\n"
        mif += "  stopping_mxHxm 0.01\n"
        mif += "  mesh :{}\n".format(meshname)
        mif += "  Ms {\n"
        mif += "    Oxs_VecMagScalarField {\n"
        mif += "      field :m0file\n"
        mif += "    }\n"
        mif += "  }\n"
        mif += "  m0 :m0file\n"
        mif += "  basename {}\n".format(systemname)
        mif += "  scalar_field_output_format {text %\#.15g}\n"
        mif += "  vector_field_output_format {text %\#.15g}\n"
        mif += "}\n\n"
        mif += "Destination table mmArchive\n"
        mif += "Destination mags mmArchive\n\n"
        mif += "Schedule DataTable table Stage 1\n"
        mif += "Schedule Oxs_MinDriver::Magnetization mags Stage 1"

        return mif

    def _check_args(self, **kwargs):
        pass



import textwrap
import discretisedfield as df


class Mesh(df.Mesh):
    @property
    def _script(self):
        mif = "# BoxAtlas\n"
        mif += "Specify Oxs_BoxAtlas:atlas {\n"
        mif += "  xrange {{{} {}}}\n".format(self.pmin[0], self.pmax[0])
        mif += "  yrange {{{} {}}}\n".format(self.pmin[1], self.pmax[1])
        mif += "  zrange {{{} {}}}\n".format(self.pmin[2], self.pmax[2])
        mif += "  name atlas\n"
        mif += "}\n\n"

        if self.pbc is not None:
            mif += "# PeriodicRectangularMesh\n"
            mif += "Specify Oxs_PeriodicRectangularMesh:{} {{\n".format(self.name)
            mif += "  cellsize {{{} {} {}}}\n".format(*self.cell)
            mif += "  atlas Oxs_BoxAtlas:atlas\n"
            mif += "  periodic {}\n".format("".join(sorted(self.pbc)))
            mif += "}\n\n"
        else:
            mif += "# RectangularMesh\n"
            mif += "Specify Oxs_RectangularMesh:{} {{\n".format(self.name)
            mif += "  cellsize {{{} {} {}}}\n".format(*self.cell)
            mif += "  atlas Oxs_BoxAtlas:atlas\n"
            mif += "}\n\n"

        return mif



import os
import pytest
import oommfc as oc


def test_example_bar():
    # can we create the object?
    b = oc.examples.bar()

    # can it do something?
    _ = b.m.average
    assert b.name == 'example-bar'

    for interaction in ['exchange', 'demag']:
        assert interaction in repr(b.hamiltonian).lower()

    for dynamics in ['precession', 'damping']:
        assert dynamics in repr(b.dynamics).lower()


def test_example_macrospin():
    # can we create the object?
    m = oc.examples.macrospin()

    # can it do something?
    _ = m.m.average
    assert m.name == 'example-macrospin'

    assert 'zeeman' in repr(m.hamiltonian).lower()

    for dynamics in ['precession', 'damping']:
        assert dynamics in repr(m.dynamics).lower()



import oommfc as oc
import discretisedfield as df


def bar():
    system = oc.System(name="example-bar")
    shape = (100e-9, 30e-9, 30e-9)
    d = 10e-9
    mesh = oc.Mesh(p1=(0, 0, 0), p2=shape, cell=(d, d, d))
    # Permalloy
    A = 1e-12
    H = (0, 0, 0)  # no Zeeman field, but provide interaction as convenience

    system.hamiltonian = oc.Exchange(A=A) + oc.Demag() + oc.Zeeman(H=H)
    alpha = 0.2
    system.dynamics = oc.Precession(gamma=oc.gamma0) + oc.Damping(alpha=alpha)
    Ms = 8e6  # A/m
    system.m = df.Field(mesh, value=(1, 0, 1), norm=Ms)

    return system



import os
import glob
import shutil
import pytest
import scipy.fftpack
import numpy as np
import oommfc as oc
import discretisedfield as df


@pytest.mark.oommf
def test_stdprobfmr():
    name = "stdprobfmr"

    # Remove any previous simulation directories.
    if os.path.exists(name):
        shutil.rmtree(name)

    lx = ly = 120e-9  # x and y dimensions of the sample(m)
    lz = 10e-9  # sample thickness (m)
    dx = dy = dz = 10e-9  # discretisation in x, y, and z directions (m)

    Ms = 8e5  # saturation magnetisation (A/m)
    A = 1.3e-11  # exchange energy constant (J/m)
    H = 8e4 * np.array([0.81345856316858023, 0.58162287266553481, 0.0])
    alpha = 0.008  # Gilbert damping
    gamma = 2.211e5

    mesh = oc.Mesh(p1=(0, 0, 0), p2=(lx, ly, lz), cell=(dx, dy, dz))

    system = oc.System(name="stdprobfmr")

    system.hamiltonian = oc.Exchange(A) + oc.Demag() + oc.Zeeman(H)
    system.dynamics = oc.Precession(gamma) + oc.Damping(alpha)
    system.m = df.Field(mesh, value=(0, 0, 1), norm=Ms)

    md = oc.MinDriver()
    md.drive(system)

    H = 8e4 * np.array([0.81923192051904048, 0.57346234436332832, 0.0])
    system.hamiltonian.zeeman.H = H

    T = 20e-9
    n = 4000

    td = oc.TimeDriver()
    td.drive(system, t=T, n=n)

    t = system.dt['t'].values
    my = system.dt['mx'].values

    psd = np.log10(np.abs(scipy.fftpack.fft(my))**2)
    f_axis = scipy.fftpack.fftfreq(4000, d=20e-9/4000)

    shutil.rmtree(name)



from .oommf import get_oommf_runner
from .hamiltonian import Exchange, UniaxialAnisotropy, \
    Demag, Zeeman, DMI, CubicAnisotropy, Hamiltonian
from .dynamics import Precession, Damping, STT, Dynamics
from .drivers import Driver, MinDriver, TimeDriver, HysteresisDriver
from .mesh import Mesh
from .system import System
from .data import Data
from micromagneticmodel.consts import mu0, e, me, kB, h, g, \
    hbar, gamma, muB, gamma0
from . import examples


def test():
    """Runs all the tests"""
    get_oommf_runner()  # pragma: no cover
    import pytest  # pragma: no cover
    args = ["-m", "not travis", "-v", "--pyargs",
            "oommfc"]  # pragma: no cover
    return pytest.main(args)  # pragma: no cover


def test_not_oommf():
    """ Runs tests that do not need an OOMMF installation"""
    import pytest  # pragma: no cover
    args = ["-m", "not travis", "-m", "not oommf", "-v",
            "--pyargs", "oommfc"]  # pragma: no cover
    return pytest.main(args)  # pragma: no cover


def test_oommf():
    """Runs all tests that require an OOMMF installation."""
    get_oommf_runner()  # pragma: no cover
    import pytest  # pragma: no cover
    args = ["-m", "oommf and not travis", "-v",
            "--pyargs", "oommfc"]  # pragma: no cover
    return pytest.main(args)  # pragma: no cover


def test_oommf_overhead(t=1e-12):
    """Run a macrospin example for time t, return system object.

    returns (time, mifpath) with
      - time : real time it took to call oommf (via Timedriver)
      - mifpath : the path to the mif file used (so that this can be re-used)

    Additional information will be printed.

    This can be used to measure/test the performance overhead of calling OOMMF.
    """
    import os
    import time

    system = examples.macrospin()

    td = TimeDriver()
    start = time.time()
    td.drive(system, t=0.001e-9, n=1)
    stop = time.time()
    time_ = stop - start
    print("Duration of calling OOMMF through oommfc: {:.4}s".format(time_))
    print("oommfc.oommf.get_oommf_runner(): {}".format(get_oommf_runner()))
    mifpath = os.path.realpath('example-macrospin/example-macrospin.mif')
    return time_, mifpath



import os
import pytest
import oommfc as oc


@pytest.mark.oommf
def test_test_oommf_overhead():
    # can we create the object?
    time_, mifpath = oc.test_oommf_overhead()

    assert isinstance(time_, float)
    assert isinstance(mifpath, str)
    assert os.path.exists(mifpath)



import sys
import micromagneticmodel as mm


class DMI(mm.DMI):
    @property
    def _script(self):
        if self.crystalclass in ["t", "o"] and sys.platform != "win32":
            mif = "# DMI of crystallographic class T(O)\n"
            mif += "Specify Oxs_DMI_T {\n"
        elif self.crystalclass == "cnv" and sys.platform != "win32":
            mif = "# DMI of crystallographic class Cnv\n"
            mif += "Specify Oxs_DMI_Cnv {\n"
        elif self.crystalclass == "d2d" and sys.platform != "win32":
            mif = "# DMI of crystallographic class D2d\n"
            mif += "Specify Oxs_DMI_D2d {\n"
        elif self.crystalclass == "cnv" and sys.platform == "win32":
            mif = "# InterfacialDMI\n"
            mif += "Specify Oxs_DMExchange6Ngbr {\n"
        else:
            raise ValueError("The {} crystal class is not supported "
                             "on {} pltform.".format(self.crystalclass, sys.platform))
        mif += "  default_D {}\n".format(self.D)
        mif += "  atlas :atlas\n"
        mif += "  D {\n"
        mif += "    atlas atlas {}\n".format(self.D)
        mif += "  }\n"
        mif += "}\n\n"

        return mif



import micromagneticmodel as mm


class UniaxialAnisotropy(mm.UniaxialAnisotropy):
    @property
    def _script(self):
        mif = "# UniaxialAnisotropy\n"
        if self.K2 == 0:
            mif += "Specify Oxs_UniaxialAnisotropy {\n"
            mif += "  K1 {}\n".format(self.K1)
            mif += "  axis {{{} {} {}}}\n".format(*self.u)
            mif += "}\n\n"
        else:
            mif += "Specify Southampton_UniaxialAnisotropy4 {\n"
            mif += "  K1 {}\n".format(self.K1)
            mif += "  K2 {}\n".format(self.K2)
            mif += "  axis {{{} {} {}}}\n".format(*self.u)
            mif += "}\n\n"

        return mif



import oommfc as oc
import micromagneticmodel.tests as mmt


class TestZeeman(mmt.TestZeeman):
    def test_script(self):
        for H in self.valid_args:
            zeeman = oc.Zeeman(H=H)

            script = zeeman._script
            assert script.count("\n") == 10
            assert script[0] == "#"
            assert script[-1] == "\n"

            lines = script.split("\n")
            assert len(lines) == 11
            assert lines[0] == "# FixedZeeman"
            assert lines[1] == "Specify Oxs_FixedZeeman {"
            assert lines[2] == "  field {"
            assert lines[3] == "    Oxs_UniformVectorField {"
            assert lines[4] == "      vector {{{} {} {}}}".format(*H)
            assert lines[5] == "    }"
            assert lines[6] == "  }"
            assert lines[7] == "  multiplier 1"
            assert lines[8] == "}"



import datetime
import logging
import os
import sys
import time
from shutil import which
from subprocess import run, PIPE

log = logging.getLogger(__name__)

class OOMMFRunner:
    """Base class for running OOMMF.
    
    Don't use this directly. You should normally use get_oommf_runner() to pick
    a subclass of this class.
    """

    def call(self, argstr, need_stderr=False):
        # print day and time at which we start calling OOMMF (useful
        # for longer runs)
        x = datetime.datetime.now()
        timestamp = "{}/{}/{} {}:{}".format(x.year, x.month, x.day,
                                            x.hour, x.minute)
        print("{}: Calling OOMMF ({}) ... ".format(timestamp, argstr), end='')

        # measure execution time of OOMMF
        tic = time.time()
        val = self._call(argstr=argstr, need_stderr=need_stderr)

        toc = time.time()
        seconds = "[{:0.1f}s]".format(toc - tic)
        print(seconds)

        # check exit code
        if val.returncode is not 0:
            stderr = val.stderr.decode('utf-8', 'replace')
            stdout = val.stdout.decode('utf-8', 'replace')
            cmdstr = " ".join(val.args)
            print("Error when executing:")
            print("\tcommand: {}".format(cmdstr))
            print("\tstdout: {}".format(stdout))
            print("\tstderr: {}".format(stderr))
            print("\n")
            raise RuntimeError("Some problem calling OOMMF.")

        return val

    def _call(self, argstr, need_stderr=False):
        # Implement in subclass
        raise NotImplementedError

    def version(self, where=None):
        p = self.call(argstr="+version", need_stderr=True)
        return p.stderr.decode('utf-8').split("oommf.tcl")[-1].strip()

    def platform(self, where=None):
        p = self.call(argstr="+platform", need_stderr=True)
        return p.stderr.decode('utf-8')

    def kill(self, targets=('all',)):
        raise NotImplementedError

class ScriptOOMMFRunner(OOMMFRunner):
    """Run OOMMF on this system, using an oommf executable on $PATH 
    """
    def __init__(self, script_name='oommf'):
        self.script_name = script_name

    def _oommf_cmd_tuple(self):
        return (self.script_name, )

    def _call(self, argstr, need_stderr=False):
        cmd = (self.script_name, "boxsi", "+fg", argstr, "-exitondone", "1")
        return run(cmd, stdout=PIPE, stderr=PIPE)

    def kill(self, targets=('all',)):
        run((self.script_name, "killoommf") + targets)


class NativeOOMMFRunner(ScriptOOMMFRunner):
    """Run OOMMF on this system, given a path to oommf.tcl
    
    This requires tclsh to be available.
    """
    def __init__(self, oommf_tcl_path):
        self.oommf_tcl_path = oommf_tcl_path

    def _call(self, argstr, need_stderr=False):
        cmd = ("tclsh", self.oommf_tcl_path, "boxsi", "+fg",
               argstr, "-exitondone", "1")
        if sys.platform == 'win32':
            stdout = stderr = None
            if need_stderr:
                stderr = PIPE
        else:
            stdout = stderr = PIPE
        return run(cmd, stdout=stdout, stderr=stderr)

    def kill(self, targets=('all',)):
        run(("tclsh", self.oommf_tcl_path, "killoommf") + targets)


class DockerOOMMFRunner(OOMMFRunner):
    """Run OOMMF inside a docker image"""
    def __init__(self, image="joommf/oommf", docker_exe='docker'):
        self.image = image
        self.docker_exe = docker_exe

    def _call(self, argstr, need_stderr=False):
        run([self.docker_exe, "pull", self.image])
        cmd = [self.docker_exe, 'run', '-v', os.getcwd()+':/io', self.image,
               "/bin/bash", "-c",
               ("tclsh /usr/local/oommf/oommf/oommf.tcl boxsi +fg {} "
                "-exitondone 1").format(argstr)]
        return run(cmd, stdout=PIPE, stderr=PIPE)

    def kill(self):
        pass # Does this need to do anything?

_cached_oommf_runner = None

def get_oommf_runner(use_cache=True, docker_exe='docker', oommf_exe='oommf'):
    """Find the best available way to run OOMMF.
    
    Returns an OOMMFRunner object, or raises EnvironmentError if no suitable
    method is found.
    
    Parameters
    ----------
    use_cache : bool
      The first call to this function will determine the best way to run OOMMF
      and cache it. Normally, subsequent calls will return the OOMMFRunner
      object from the cache. Setting this parameter to False will cause it to
      check for available methods again.
    docker_exe : str
      The name or path of the docker command.
    """
    global _cached_oommf_runner
    if use_cache and (_cached_oommf_runner is not None):
        return _cached_oommf_runner

    # Check for $OOMMFTCL environment variable pointing to native OOMMF
    oommf_tcl_path = os.environ.get('OOMMFTCL', None)
    if oommf_tcl_path:
        cmd = ("tclsh", oommf_tcl_path, "boxsi",
               "+fg", "+version", "-exitondone", "1")
        try:
            res = run(cmd, stdout=PIPE, stderr=PIPE)
        except FileNotFoundError:
            log.warning("tclsh was not found")
        else:
            if res.returncode:
                log.warning("OOMMFTCL is set, but there was a problem running oommf.\n"
                            "stdout:\n{}\n\n"
                            "stderr:\n{}".format(res.stdout, res.stderr))
            else:
                _cached_oommf_runner = NativeOOMMFRunner(oommf_tcl_path)
                return _cached_oommf_runner

    if sys.platform == 'win32' and \
            os.path.isdir(os.path.join(sys.prefix, 'conda-meta')):
        # In a conda env on Windows, would probably also work on Mac/Linux
        oommf_tcl = os.path.join(sys.prefix, 'opt', 'oommf', 'oommf.tcl')
        if os.path.isfile(oommf_tcl):
            _cached_oommf_runner = NativeOOMMFRunner(oommf_tcl)
            return _cached_oommf_runner

	# 'oommf' available as a command - in a conda env on Mac/Linux, or oommf installed separately
    oommf_exe_path = which(oommf_exe)
    if oommf_exe_path:
        _cached_oommf_runner = ScriptOOMMFRunner(oommf_exe_path)
        return _cached_oommf_runner

    # Check for docker to run OOMMF in a docker image
    cmd = (docker_exe, "images")
    try:
        res = run(cmd, stdout=PIPE, stderr=PIPE)
    except FileNotFoundError:
        log.warning("docker was not found")
    else:
        if res.returncode:
            log.warning("Error running docker\n"
                "stdout:\n{}\n\n"
                "stderr:\n{}".format(res.stdout, res.stderr))
        else:
            _cached_oommf_runner = DockerOOMMFRunner(image="joommf/oommf")
            return _cached_oommf_runner

    # Raise exception if we can't find a way to run OOMMF
    raise EnvironmentError("Could not run $OOMMFTCL or docker.")



import os
import glob
import shutil
import pytest
import oommfc as oc
from .test_driver import TestDriver


class TestTimeDriver(TestDriver):
    def test_script(self):
        driver = oc.TimeDriver()
        t = 1e-9
        n = 120
        script = driver._script(self.system, t=t, n=n)

        assert script[0] == "#"
        assert script[-1] == "1"
        assert script.count("#") == 5
        assert script.count("Specify") == 3
        assert script.count("Destination") == 3
        assert script.count("Schedule") == 2
        assert script.count("mmArchive") == 3
        assert script.count("Stage") == 2

        assert "Oxs_RungeKuttaEvolve" in script
        assert "Oxs_TimeDriver" in script
        assert "Oxs_FileVectorField" in script

        lines = script.split("\n")
        alpha = self.system.dynamics.damping.alpha
        assert lines[2] == "  alpha {}".format(alpha)
        gamma = self.system.dynamics.precession.gamma
        assert lines[3] == "  gamma_G {}".format(gamma)

    def test_save_mif(self):
        driver = oc.TimeDriver()
        t = 1e-9
        n = 120
        driver._makedir(self.system)
        driver._save_mif(self.system, t=t, n=n)

        miffilename = os.path.join("tds", "tds.mif")
        assert os.path.isfile(miffilename)

        lines = open(miffilename, "r").readlines()
        assert lines[0] == "# MIF 2.1\n"
        assert lines[-1][-1] == "1"

        shutil.rmtree("tds")

    @pytest.mark.oommf
    def test_drive(self):
        md = oc.TimeDriver()

        md.drive(self.system, t=0.1e-9, n=10)

        assert os.path.exists("tds")
        miffilename = os.path.join("tds", "tds.mif")
        assert os.path.isfile(miffilename)

        omf_files = list(glob.iglob("tds/*.omf"))
        odt_files = list(glob.iglob("tds/*.odt"))

        assert len(omf_files) == 11
        omffilename = os.path.join("tds", "m0.omf")
        assert omffilename in omf_files

        assert len(odt_files) == 1

        shutil.rmtree("tds")

    def test_drive_exception(self):
        md = oc.TimeDriver()
        with pytest.raises(ValueError):
            md.drive(self.system, t=-0.1e-9, n=10)
        with pytest.raises(ValueError):
            md.drive(self.system, t=0.1e-9, n=-10)

    def test_script_missing_terms(self):
        # Missing Gilbert damping alpha
        self.system.dynamics = oc.Precession(gamma=2.2)
        td = oc.TimeDriver()
        script = td._script(self.system, t=1e-9, n=20)
        assert "alpha 0" in script
        assert "gamma_G 2.2" in script

        # Missing gamma
        self.system.dynamics = oc.Damping(alpha=5)
        td = oc.TimeDriver()
        script = td._script(self.system, t=1e-9, n=20)
        assert "alpha 5" in script
        assert "gamma_G 221100" in script



import sys
import pytest
import shutil
import testpath
import subprocess
from oommfc.oommf import DockerOOMMFRunner, get_oommf_runner

nonexistant_docker = "docker-executable-name-like-this-does-not-exist"

@pytest.mark.travis
def test_exception_is_raised_if_no_docker():
    runner = DockerOOMMFRunner(docker_exe=nonexistant_docker)

    # expect exception as docker executable doesn't exist:
    with pytest.raises(FileNotFoundError):
        runner.call(argstr="+version")

@pytest.mark.travis
def test_docker_installed_not_running():
    if not shutil.which('docker'):
        pytest.skip('docker command not found')
    status, output = subprocess.getstatusoutput('docker ps')
    if status == 0:
        pytest.skip("Docker appears to be running.")

    # expect "Cannot connect to the Docker daemon. Is the docker
    # daemon running on this host?'"
    assert 'Cannot connect to the Docker daemon' in output

    runner = DockerOOMMFRunner()
    with pytest.raises(RuntimeError):
        runner.call(argstr="+version")

@pytest.mark.travis
def test_no_runner_found():
    # Check that we get EnvironmentError if neither OOMMF nor docker are found
    if sys.platform == 'win32' and (
            ('Continuum' in sys.version) or ('Anaconda' in sys.version)):
        pytest.skip("Can't prevent finding oommmf in windows conda env")
    with testpath.modified_env({'OOMMFTCL': None}):
        with pytest.raises(EnvironmentError):
            get_oommf_runner(use_cache=False, docker_exe=nonexistant_docker,
                             oommf_exe=nonexistant_docker)



import os
import glob
import shutil
import pytest
import numpy as np
import oommfc as oc
import discretisedfield as df
from scipy.optimize import bisect


@pytest.mark.oommf
def test_stdprob5():
    name = "stdprob5"

    # Remove any previous simulation directories.
    if os.path.exists(name):
        shutil.rmtree(name)

    # Geometry
    lx = 100e-9  # x dimension of the sample(m)
    ly = 100e-9  # y dimension of the sample (m)
    lz = 10e-9  # sample thickness (m)

    # Material (permalloy) parameters
    Ms = 8e5  # saturation magnetisation (A/m)
    A = 1.3e-11  # exchange energy constant (J/m)

    # Dynamics (LLG + STT equation) parameters
    gamma = 2.211e5  # gyromagnetic ratio (m/As)
    alpha = 0.1  # Gilbert damping
    ux = -72.35  # velocity in x direction
    beta = 0.05  # non-adiabatic STT parameter

    system = oc.System(name=name)
    mesh = oc.Mesh(p1=(0, 0, 0), p2=(100e-9, 100e-9, 10e-9),
                   cell=(5e-9, 5e-9, 5e-9))
    system.hamiltonian = oc.Exchange(A) + oc.Demag()

    def m_vortex(pos):
        x, y, z = pos[0]/1e-9-50, pos[1]/1e-9-50, pos[2]/1e-9
        return (-y, x, 10)

    system.m = df.Field(mesh, value=m_vortex, norm=Ms)

    md = oc.MinDriver()
    md.drive(system)

    system.dynamics += oc.Precession(gamma) + oc.Damping(alpha) + \
        oc.STT(u=(ux, 0, 0), beta=beta)

    td = oc.TimeDriver()
    td.drive(system, t=8e-9, n=100)

    mx = system.dt["mx"].values

    assert -0.03 < mx.max() < 0
    assert -0.35 < mx.min() < -0.30

    shutil.rmtree(name)



import micromagneticmodel as mm


class Zeeman(mm.Zeeman):
    @property
    def _script(self):
        mif = '# FixedZeeman\n'
        mif += 'Specify Oxs_FixedZeeman {\n'
        mif += '  field {\n'
        mif += '    Oxs_UniformVectorField {\n'
        mif += '      vector {{{} {} {}}}\n'.format(*self.H)
        mif += '    }\n'
        mif += '  }\n'
        mif += '  multiplier 1\n'
        mif += '}\n\n'

        return mif



import os
import shutil
import pytest
import oommfc as oc
import discretisedfield as df
import micromagneticmodel.tests as mmt


class TestCubicAnisotropy(mmt.TestCubicAnisotropy):
    def test_script(self):
        for K1, u1, u2 in self.valid_args:
            anisotropy = oc.CubicAnisotropy(K1=K1, u1=u1, u2=u2)
            script = anisotropy._script

            assert script.count("\n") == 7
            assert script[0] == "#"
            assert script[-1] == "\n"
            lines = script.split("\n")
            assert len(lines) == 8
            assert lines[0] == "# CubicAnisotropy"
            assert lines[2] == "  K1 {}".format(K1)
            assert lines[3] == "  axis1 {{{} {} {}}}".format(*u1)
            assert lines[4] == "  axis2 {{{} {} {}}}".format(*u2)
            assert lines[5] == "}"


@pytest.mark.oommf
def test_relax_with_cubicanisotropy():
    name = "cubic_anisotropy"
    L = 100e-9
    d = 5e-9
    Ms = 8e6

    # Remove any previous simulation directories.
    if os.path.exists(name):
        shutil.rmtree(name)

    system = oc.System(name=name)
    system.hamiltonian = oc.CubicAnisotropy(K1=5e6,
                                            u1=(1, 0, 0),
                                            u2=(0, 1, 0))

    mesh = oc.Mesh(p1=(0, 0, 0), p2=(L, L, L), cell=(d, d, d))

    def m_init(pos):
        x, y, z = pos
        if x < 30e-9:
            return (0.7, 0.1, 0.3)
        elif x > 70e-9:
            return (0.1, 0.7, 0.3)
        else:
            return (0.3, 0.1, 0.7)

    system.m = df.Field(mesh, value=m_init, norm=Ms)

    md = oc.MinDriver()
    md.drive(system)

    comp_value = 0.99*Ms
    assert system.m((10e-9, 0, 0))[0] > comp_value
    assert system.m((50e-9, 0, 0))[2] > comp_value
    assert system.m((80e-9, 0, 0))[1] > comp_value

    shutil.rmtree(name)



import oommfc as oc
import micromagneticmodel.tests as mmt


class TestExchange(mmt.TestExchange):
    def test_script(self):
        for A in self.valid_args:
            exchange = oc.Exchange(A)

            script = exchange._script
            assert script.count("\n") == 5
            assert script[0] == "#"
            assert script[-1] == "\n"

            lines = script.split("\n")
            assert len(lines) == 6
            assert lines[0] == "# UniformExchange"
            assert lines[1] == "Specify Oxs_UniformExchange {"
            assert lines[2] == "  A {}".format(A)
            assert lines[3] == "}"



import os
import shutil
import oommfc as oc
import discretisedfield as df


def test_skyrmion():
    name = "skyrmion"

    # Remove any previous simulation directories.
    if os.path.exists(name):
        shutil.rmtree(name)

    mesh = oc.Mesh(p1=(-50e-9, -50e-9, 0),
                   p2=(50e-9, 50e-9, 10e-9),
                   cell=(5e-9, 5e-9, 5e-9))

    system = oc.System(name="skyrmion")
    system.hamiltonian = oc.Exchange(A=1.6e-11) + \
        oc.DMI(D=4e-3, crystalclass="cnv") + \
        oc.UniaxialAnisotropy(K1=0.51e6, K2=0.1, u=(0, 0, 1)) + \
        oc.Demag() + \
        oc.Zeeman(H=(0, 0, 2e5))

    Ms = 1.1e6

    def Ms_fun(pos):
        x, y, z = pos
        if (x**2 + y**2)**0.5 < 50e-9:
            return Ms
        else:
            return 0

    def m_init(pos):
        x, y, z = pos
        if (x**2 + y**2)**0.5 < 10e-9:
            return (0, 0.1, -1)
        else:
            return (0, 0.1, 1)

    system.m = df.Field(mesh, value=m_init, norm=Ms_fun)

    md = oc.MinDriver()
    md.drive(system)

    # Check the magnetisation at the sample centre.
    assert system.m((0, 0, 0))[2]/Ms < -0.97

    # Check the magnetisation at the sample edge.
    assert system.m((50e-9, 0, 0))[2]/Ms > 0

    shutil.rmtree(name)



from .driver import Driver


class TimeDriver(Driver):
    def _script(self, system, **kwargs):
        try:
            alpha = system.dynamics.damping.alpha
        except AttributeError:
            alpha = 0
        try:
            gamma = system.dynamics.precession.gamma
        except AttributeError:
            gamma = 2.211e5
        try:
            u = system.dynamics.stt.u
            beta = system.dynamics.stt.beta
        except AttributeError:
            stt = False
        else:
            stt = True

        meshname = system.m.mesh.name
        systemname = system.name
        if "derive" in kwargs:
            t, n = 1e-20, 1
        else:
            t, n = kwargs["t"], kwargs["n"]

        if not stt:
            mif = "# RungeKuttaEvolver\n"
            mif += "Specify Oxs_RungeKuttaEvolve:evolver {\n"
            mif += "  alpha {}\n".format(alpha)
            mif += "  gamma_G {}\n".format(gamma)
            mif += "}\n\n"
            evolver = "Oxs_RungeKuttaEvolve"
        else:
            mif = "# SpinTransferTorqueEvolver\n"
            mif += "Specify Anv_SpinTEvolve {\n"
            mif += "  do_precess 1\n"
            mif += "  gamma_G {}\n".format(gamma)
            mif += "  alpha {}\n".format(alpha)
            mif += "  method rkf54s\n"
            mif += "  u {\n"
            mif += "    Oxs_UniformScalarField {\n"
            mif += "      value {}\n".format(u[0])
            mif += "    }\n"
            mif += "  }\n"
            mif += "  beta {}\n".format(beta)
            mif += "}\n\n"
            evolver = "Anv_SpinTEvolve"

        mif += "# m0 file\n"
        mif += "Specify Oxs_FileVectorField:m0file {\n"
        mif += "   atlas :atlas\n"
        mif += "   file m0.omf\n"
        mif += "}\n\n"

        mif += "# TimeDriver\n"
        mif += "Specify Oxs_TimeDriver {\n"
        mif += "  evolver {}\n".format(evolver)
        mif += "  stopping_time {}\n".format(t/n)
        mif += "  mesh :{}\n".format(meshname)
        mif += "  stage_count {}\n".format(n)
        if "total_iteration_limit" in kwargs:
            mif += "  total_iteration_limit {}\n".format(total_iteration_limit)
        elif "derive" in kwargs:
            mif += "  total_iteration_limit {}\n".format(1)

        mif += "  Ms {\n"
        mif += "    Oxs_VecMagScalarField {\n"
        mif += "      field :m0file\n"
        mif += "    }\n"
        mif += "  }\n"
        mif += "  m0 :m0file\n"
        mif += "  basename {}\n".format(systemname)
        mif += "  scalar_field_output_format {text %\#.15g}\n"
        mif += "  vector_field_output_format {text %\#.15g}\n"
        mif += "}\n\n"
        mif += "Destination table mmArchive\n"
        mif += "Destination mags mmArchive\n"
        mif += "Destination archive mmArchive\n\n"
        if "derive" in kwargs:
            if "ield" in kwargs["derive"] or "density" in kwargs["derive"]:
                mif += ("Schedule \"{}\" archive "
                        "Step 1".format(kwargs["derive"]))
            else:
                mif += "Schedule DataTable table Stage 1\n"
        else:
            mif += "Schedule DataTable table Stage 1\n"
            mif += "Schedule Oxs_TimeDriver::Magnetization mags Stage 1"

        return mif

    def _check_args(self, **kwargs):
        if "derive" in kwargs:
            pass
        else:
            if kwargs["t"] <= 0:
                raise ValueError("Expected t > 0.")
            if kwargs["n"] <= 0 or not isinstance(kwargs["n"], int):
                raise ValueError("Expected n > 0.")



import oommfc as oc
import discretisedfield.tests as dft


class TestMesh(dft.TestMesh):
    def test_script_no_pbc(self):
        for p1, p2, cell in self.valid_args:
            name = "test_mesh"
            mesh = oc.Mesh(p1, p2, cell, name=name)

            script = mesh._script
            assert script.count("\n") == 14
            assert script[0] == "#"
            assert script[-1] == "\n"

            lines = script.split("\n")
            assert len(lines) == 15

            # Assert BoxAtlas script
            assert lines[0] == "# BoxAtlas"
            assert lines[1] == "Specify Oxs_BoxAtlas:atlas {"
            assert lines[2] == "  xrange {{{} {}}}".format(mesh.pmin[0],
                                                           mesh.pmax[0])
            assert lines[3] == "  yrange {{{} {}}}".format(mesh.pmin[1],
                                                           mesh.pmax[1])
            assert lines[4] == "  zrange {{{} {}}}".format(mesh.pmin[2],
                                                           mesh.pmax[2])
            assert lines[5] == "  name atlas"
            assert lines[6] == "}"

            # Empty line between BoxAtlas and RectangularMesh
            assert lines[7] == ""

            # Assert RectangularMesh script
            assert lines[8] == "# RectangularMesh"
            assert lines[9] == "Specify Oxs_RectangularMesh:{} {{".format(name)
            assert lines[10] == "  cellsize {{{} {} {}}}".format(*cell)
            assert lines[11] == "  atlas Oxs_BoxAtlas:atlas"
            assert lines[12] == "}"

    def test_script_pbc(self):
        for p1, p2, cell in self.valid_args:
            pbc = "xy"
            name = "test_mesh"
            mesh = oc.Mesh(p1, p2, cell, pbc=pbc, name=name)

            script = mesh._script
            assert script.count("\n") == 15
            assert script[0] == "#"
            assert script[-1] == "\n"

            lines = script.split("\n")
            assert len(lines) == 16

            # Assert BoxAtlas script
            assert lines[0] == "# BoxAtlas"
            assert lines[1] == "Specify Oxs_BoxAtlas:atlas {"
            assert lines[2] == "  xrange {{{} {}}}".format(mesh.pmin[0],
                                                           mesh.pmax[0])
            assert lines[3] == "  yrange {{{} {}}}".format(mesh.pmin[1],
                                                           mesh.pmax[1])
            assert lines[4] == "  zrange {{{} {}}}".format(mesh.pmin[2],
                                                           mesh.pmax[2])
            assert lines[5] == "  name atlas"
            assert lines[6] == "}"

            # Empty line between BoxAtlas and RectangularMesh
            assert lines[7] == ""

            # Assert RectangularMesh script
            assert lines[8] == "# PeriodicRectangularMesh"
            assert lines[9] == ("Specify Oxs_PeriodicRectangularMesh:{} "
                                "{{".format(name))
            assert lines[10] == "  cellsize {{{} {} {}}}".format(*cell)
            assert lines[11] == "  atlas Oxs_BoxAtlas:atlas"
            assert lines[12] == "  periodic xy"
            assert lines[13] == "}"



import micromagneticmodel as mm


class CubicAnisotropy(mm.CubicAnisotropy):
    @property
    def _script(self):
        mif = "# CubicAnisotropy\n"
        mif += "Specify Oxs_CubicAnisotropy {\n"
        mif += "  K1 {}\n".format(self.K1)
        mif += "  axis1 {{{} {} {}}}\n".format(*self.u1)
        mif += "  axis2 {{{} {} {}}}\n".format(*self.u2)
        mif += "}\n\n"

        return mif



import oommfc as oc
import micromagneticmodel.tests as mmt


class TestUniaxialAnisotropy(mmt.TestUniaxialAnisotropy):
    def test_script(self):
        for K1, K2, u in self.valid_args:
            anisotropy = oc.UniaxialAnisotropy(K1=K1, K2=K2, u=u)
            script = anisotropy._script

            assert script[0] == "#"
            assert script[-1] == "\n"
            lines = script.split("\n")
            assert lines[0] == "# UniaxialAnisotropy"
            assert lines[2] == "  K1 {}".format(K1)
            assert lines[-4] == "  axis {{{} {} {}}}".format(*u)
            assert lines[-3] == "}"

            if K2 == 0:
                assert script.count("\n") == 6
                assert len(lines) == 7
                assert lines[1] == "Specify Oxs_UniaxialAnisotropy {"
            else:
                assert script.count("\n") == 7
                assert len(lines) == 8
                assert lines[1] == "Specify Southampton_UniaxialAnisotropy4 {"
                assert lines[3] == "  K2 {}".format(K2)



import os
import glob
import oommfodt
import micromagneticmodel as mm
import discretisedfield as df
import oommfc as oc


class Driver(mm.Driver):
    def drive(self, system, **kwargs):
        """
        Drive the system.

        """
        self._check_args(**kwargs)

        filenames = self._filenames(system)

        # Make a directory for saving OOMMF files.
        self._makedir(system)

        # Save system's magnetisation configuration omf file.
        omffilename = filenames["omffilename"]
        system.m.write(omffilename)

        miffilename = filenames["miffilename"]
        self._save_mif(system, **kwargs)

        self._run_simulator(system)

        if "derive" not in kwargs:
            self._update_system(system)

    def _makedir(self, system):
        """
        Create directory where OOMMF files are saved.
        """
        dirname = self._filenames(system)["dirname"]
        if not os.path.exists(dirname):
            os.makedirs(dirname)

    def _save_mif(self, system, **kwargs):
        """
        Save OOMMF configuration mif file.
        """
        mif = "# MIF 2.1\n\n"
        mif += system._script
        mif += self._script(system, **kwargs)

        miffilename = self._filenames(system)["miffilename"]
        miffile = open(miffilename, "w")
        miffile.write(mif)
        miffile.close()

    def _run_simulator(self, system):
        miffilename = self._filenames(system)["miffilename"]
        oommf = oc.get_oommf_runner()
        oommf.call(argstr=miffilename)
        oommf.kill()

    def _update_system(self, system):
        self._update_m(system)
        self._update_dt(system)

    def _update_m(self, system):
        # Find last omf file.
        dirname = self._filenames(system)["dirname"]
        last_omf_file = max(glob.iglob("{}*.omf".format(dirname)),
                            key=os.path.getctime)

        # Update system's magnetisaton.
        m_field = df.read(last_omf_file)

        # Temporary solution for having script in mesh object.
        # Overwrites the df.Mesh with oc.Mesh.
        m_field.mesh = system.m.mesh

        system.m = m_field

    def _update_dt(self, system):
        # Find last odt file.
        dirname = self._filenames(system)["dirname"]
        last_odt_file = max(glob.iglob("{}*.odt".format(dirname)),
                            key=os.path.getctime)

        # Update system's datatable.
        system.dt = oommfodt.read(last_odt_file)

    def _filenames(self, system):
        dirname = os.path.join(system.name, "")
        omffilename = os.path.join(dirname, "m0.omf")
        miffilename = os.path.join(dirname, "{}.mif".format(system.name))

        filenames = {}
        filenames["dirname"] = dirname
        filenames["omffilename"] = omffilename
        filenames["miffilename"] = miffilename

        return filenames



import shutil
import pytest
import oommfc as oc
import discretisedfield as df


@pytest.mark.oommf
def test_stdprob1():
    name = "stdprob1"

    # Geometry
    lx = 2e-6  # x dimension of the sample(m)
    ly = 1e-6  # y dimension of the sample (m)
    lz = 20e-9  # sample thickness (m)

    # Material parameters
    Ms = 8e5  # saturation magnetisation (A/m)
    A = 1.3e-11  # exchange energy constant (J/m)
    K = 0.5e3  # uniaxial anisotropy constant (J/m**3)
    u = (1, 0, 0)  # uniaxial anisotropy axis

    # Create a mesh object.
    mesh = oc.Mesh(p1=(0, 0, 0), p2=(lx, ly, lz), cell=(20e-9, 20e-9, 20e-9))

    system = oc.System(name=name)
    system.hamiltonian = oc.Exchange(A) + oc.UniaxialAnisotropy(K, u) + \
        oc.Demag()
    system.m = df.Field(mesh, value=(-10, -1, 0), norm=Ms)

    Hmax = (50e-3/oc.mu0, 0.87275325e-3/oc.mu0, 0)
    Hmin = (-50e-3/oc.mu0, -0.87275325e-3/oc.mu0, 0)
    n = 10

    hd = oc.HysteresisDriver()
    hd.drive(system, Hmax=Hmax, Hmin=Hmin, n=n)

    Bx = system.dt["Bx"].values
    mx = system.dt["mx"].values

    assert len(mx) == 21
    assert len(Bx) == 21

    shutil.rmtree(name)



import micromagneticmodel as mm


class System(mm.System):
    """Micromagnetic system oject.

    Parameters
    ----------
    name : str

    Examples
    --------
    Creating a simple system object.

    >>> import oommfc as oc
    >>> system = oc.System(name="my_system")

    """
    @property
    def _script(self):
        mif = self.m.mesh._script
        mif += self.hamiltonian._script
        return mif

    def total_energy(self):
        return self.dt.tail(1)["E"][0]



import os
import glob
import shutil
import pytest
import oommfc as oc
from .test_driver import TestDriver


class TestHysteresisDriver(TestDriver):
    def test_script(self):
        hd = oc.HysteresisDriver()

        script = hd._script(self.system, Hmin=(0, 0, 0),
                            Hmax=(10, 10, 10), n=10)

        assert script[0] == "#"
        assert script[-1] == "1"
        assert script.count("#") == 6
        assert script.count("Specify") == 4
        assert script.count("Destination") == 2
        assert script.count("Schedule") == 2
        assert script.count("mmArchive") == 2
        assert script.count("Stage") == 2

        assert "Oxs_UZeeman" in script
        assert "Oxs_CGEvolve" in script
        assert "Oxs_MinDriver" in script
        assert "Oxs_FileVectorField" in script

    def test_save_mif(self):
        hd = oc.HysteresisDriver()

        hd._makedir(self.system)
        hd._save_mif(self.system, Hmin=(0, 0, 0), Hmax=(1, 1, 1), n=10)

        miffilename = os.path.join("tds", "tds.mif")
        assert os.path.isfile(miffilename)

        lines = open(miffilename, "r").readlines()
        assert lines[0] == "# MIF 2.1\n"
        assert lines[-1][-1] == "1"

        shutil.rmtree("tds")

    @pytest.mark.oommf
    def test_drive(self):
        hd = oc.HysteresisDriver()

        hd.drive(self.system, Hmin=(0, 0, 0), Hmax=(1, 1, 1), n=10)

        dirname = os.path.join("tds", "")
        assert os.path.exists(dirname)
        miffilename = os.path.join("tds", "tds.mif")
        assert os.path.isfile(miffilename)

        omf_files = list(glob.iglob("tds/*.omf"))
        odt_files = list(glob.iglob("tds/*.odt"))

        assert len(omf_files) == 22
        omffilename = os.path.join("tds", "m0.omf")
        assert omffilename in omf_files

        assert len(odt_files) == 1

        shutil.rmtree("tds")



import os
import glob
import oommfodt
import oommfc as oc
import discretisedfield as df
import micromagneticmodel as mm


class Data(mm.Data):
    @property
    def effective_field(self):
        _dict = {"Demag": "Oxs_Demag::Field",
                 "Exchange": "Oxs_UniformExchange::Field",
                 "UniaxialAnisotropy": "Oxs_UniaxialAnisotropy::Field",
                 "Zeeman": "Oxs_FixedZeeman::Field",
                 "Hamiltonian": "Oxs_RungeKuttaEvolve:evolver:Total field"}

        td = oc.TimeDriver()
        td.drive(self.system, derive=_dict[self.cls])

        dirname = os.path.join(self.system.name, "")
        ohf_file = max(glob.iglob("{}*.ohf".format(dirname)),
                       key=os.path.getctime)

        return df.read(ohf_file)

    @property
    def energy(self):
        _dict = {"Demag": "Demag::Energy",
                 "Exchange": "UniformExchange::Energy",
                 "UniaxialAnisotropy": "UniaxialAnisotropy::Energy",
                 "Zeeman": "FixedZeeman::Energy",
                 "Hamiltonian": "RungeKuttaEvolve:evolver:Totalenergy"}
        td = oc.TimeDriver()
        td.drive(self.system, derive="energy")

        dirname = os.path.join(self.system.name, "")
        odt_file = max(glob.iglob("{}*.odt".format(dirname)),
                       key=os.path.getctime)

        dt = oommfodt.read(odt_file, replace_columns=False)

        return dt[_dict[self.cls]][0]

    @property
    def energy_density(self):
        _dict = {"Demag": "Oxs_Demag::Energy density",
                 "Exchange": "Oxs_UniformExchange::Energy density",
                 "UniaxialAnisotropy": ("Oxs_UniaxialAnisotropy::"
                                        "Energy density"),
                 "Zeeman": "Oxs_FixedZeeman::Energy density",
                 "Hamiltonian": ("Oxs_RungeKuttaEvolve:evolver:"
                                 "Total energy density")}

        td = oc.TimeDriver()
        td.drive(self.system, derive=_dict[self.cls])

        dirname = os.path.join(self.system.name, "")
        oef_file = max(glob.iglob("{}*.oef".format(dirname)),
                       key=os.path.getctime)

        return df.read(oef_file)



import os
import glob
import shutil
import pytest
import numpy as np
import oommfc as oc
import discretisedfield as df
from scipy.optimize import bisect


@pytest.mark.oommf
def test_stdprob3():
    name = "stdprob3"

    # Remove any previous simulation directories.
    if os.path.exists(name):
        shutil.rmtree(name)

    # Function for initiaising the flower state.
    def m_init_flower(pos):
        x, y, z = pos[0]/1e-9, pos[1]/1e-9, pos[2]/1e-9
        mx = 0
        my = 2*z - 1
        mz = -2*y + 1
        norm_squared = mx**2 + my**2 + mz**2
        if norm_squared <= 0.05:
            return (1, 0, 0)
        else:
            return (mx, my, mz)

    # Function for initialising the vortex state.
    def m_init_vortex(pos):
        x, y, z = pos[0]/1e-9, pos[1]/1e-9, pos[2]/1e-9
        mx = 0
        my = np.sin(np.pi/2 * (x-0.5))
        mz = np.cos(np.pi/2 * (x-0.5))

        return (mx, my, mz)

    def minimise_system_energy(L, m_init):
        N = 16  # discretisation in one dimension
        cubesize = 100e-9  # cube edge length (m)
        cellsize = cubesize/N  # discretisation in all three dimensions.
        lex = cubesize/L  # exchange length.

        Km = 1e6  # magnetostatic energy density (J/m**3)
        Ms = np.sqrt(2*Km/oc.mu0)  # magnetisation saturation (A/m)
        A = 0.5 * oc.mu0 * Ms**2 * lex**2  # exchange energy constant
        K = 0.1*Km  # Uniaxial anisotropy constant
        u = (0, 0, 1)  # Uniaxial anisotropy easy-axis

        p1 = (0, 0, 0)  # Minimum sample coordinate.
        p2 = (cubesize, cubesize, cubesize)  # Maximum sample coordinate.
        cell = (cellsize, cellsize, cellsize)  # Discretisation.
        mesh = oc.Mesh(p1=(0, 0, 0), p2=(cubesize, cubesize, cubesize),
                       cell=(cellsize, cellsize, cellsize))

        system = oc.System(name=name)
        system.hamiltonian = oc.Exchange(A) + oc.UniaxialAnisotropy(K, u) + \
            oc.Demag()
        system.m = df.Field(mesh, value=m_init, norm=Ms)

        md = oc.MinDriver()
        md.drive(system)

        return system

    def energy_difference(L):
        vortex = minimise_system_energy(L, m_init_vortex)
        flower = minimise_system_energy(L, m_init_flower)

        return vortex.total_energy() - flower.total_energy()

    cross_section = bisect(energy_difference, 8, 9, xtol=0.1)
    assert 8.4 < cross_section < 8.5

    shutil.rmtree(name)



from .exchange import Exchange
from .uniaxialanisotropy import UniaxialAnisotropy
from .cubicanisotropy import CubicAnisotropy
from .demag import Demag
from .zeeman import Zeeman
from .dmi import DMI
from .hamiltonian import Hamiltonian



import micromagneticmodel as mm


class Hamiltonian(mm.Hamiltonian):
    @property
    def _script(self):
        mif = ""
        for term in self.terms:
            mif += term._script
        return mif
